Next {
    align: center middle;
}

Next > Container {
    width: auto;
    height: auto;
    align: center middle;
    padding: 1 2;
    background: $panel;
}

Next > Container > Label {
    width: 100%;
    content-align-horizontal: center;
}

Next > Container > Horizontal {
    height: auto;
    width: auto;
}

Next > Container > Horizontal > Button {
    margin: 1 2;
}

#hosters {
    height: auto;
}

#input {
    margin: 1 0;
}

#markdown {
    margin: 0 0;
    margin-top: -2;
}

/*TODO: make height 100 of what the table needs, so there is only one scroll*/
#season_list {
    margin: 1 0;
    margin-top: 0;
    margin-bottom: 0;
    min-height: 3;
}

TabbedContent #--content-tab-setting {
    dock: right;
    margin-right: 1;
}

#setting > RadioButton {
    margin-bottom: 1;
}

ClickableListItem {
    padding-bottom: -1;
    padding-left: -1;
    padding-right: -1;
}

#footer {
    background: $accent;
    color: $text;
    dock: bottom;
    height: 1;
}

SortableTable {
    width: auto;
}

/*$accent: lime;*/
